@@ -1,3 +1,31 @@
+# Copyright (c) 2007, Guilherme Polo.
+# All rights reserved.
+#
+# Redistribution and use in source and binary forms, with or without 
+# modification, are permitted provided that the following conditions are met:
+#
+#   1. Redistributions of source code must retain the above copyright notice, 
+#      this list of conditions and the following disclaimer.
+#
+#   2. Redistributions in binary form must reproduce the above copyright 
+#      notice, this list of conditions and the following disclaimer in the 
+#      documentation and/or other materials provided with the distribution.
+#
+#   3. The name of the author may not be used to endorse or promote products 
+#      derived from this software without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
+# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+
 """
 Storm ORM code generator.
 """
@@ -46,6 +74,8 @@ class StormConstructor(object):
         default_inheritance = '%s.Storm' % PREFIX
         classes = []
         extra_imports = []
+        # classes that inherits from others must be written at end
+        classes_inheritance = []
         
         for table in self.dtables.values():
             new_class = {'name': self._class_name(table.name), 
@@ -139,15 +169,22 @@ class StormConstructor(object):
             
             new_class['attributes'] = '\n'.join(attribs)
             
+            if new_class['inheritance'] != default_inheritance:
+                append_to = classes_inheritance
+            else:
+                append_to = classes
+                
             if not args:
-                classes.append(TMPL_CLASS_BASE % new_class)
+                append_to.append(TMPL_CLASS_BASE % new_class)
             else:
                 new_class['args'] = ', '.join(args)
                 new_class['attribs_init'] = '\n'.join(attribs_init)
-                classes.append(TMPL_CLASS % new_class)
+                append_to.append(TMPL_CLASS % new_class)
 
         extra_imports = ['import %s' % eimp for eimp in extra_imports]
         extra_imports = '\n'.join(extra_imports)
+        classes.extend(classes_inheritance)
+        
         return extra_imports, STORM_IMPORT, '\n\n'.join(classes)
     
     def _mapping(self, tomap):
