@@ -1,7 +1,36 @@
+# Copyright (c) 2007, Guilherme Polo.
+# All rights reserved.
+#
+# Redistribution and use in source and binary forms, with or without 
+# modification, are permitted provided that the following conditions are met:
+#
+#   1. Redistributions of source code must retain the above copyright notice, 
+#      this list of conditions and the following disclaimer.
+#
+#   2. Redistributions in binary form must reproduce the above copyright 
+#      notice, this list of conditions and the following disclaimer in the 
+#      documentation and/or other materials provided with the distribution.
+#
+#   3. The name of the author may not be used to endorse or promote products 
+#      derived from this software without specific prior written permission.
+#
+# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER 
+# OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+
 """
-No validation is being done yet.
+No validation is being done at all.
 """
 
+from uml2orm.consts import DEFAULT_DB_NAME
 from uml2orm.umlparser.base import BaseParser
 
 class XmiParser(BaseParser):
@@ -29,14 +58,16 @@ class XmiParser(BaseParser):
             if child.tag != self.xmi_layer:
                 continue
 
-            for c in child.getchildren()[0].getchildren()[0].getchildren():
-                local = self.nstracker.local_name(c.tag)
-                if local == "Model":
-                    for _c in c.getchildren():
-                        l = self.nstracker.local_name(_c.tag)
-                        if l == "Namespace.ownedElement":
-                            self.uml_layer = _c
-                            return
+            for obj in child.getchildren()[0].getchildren()[0].getchildren():
+                local = self.nstracker.local_name(obj.tag)
+                if local != "Model":
+                    continue
+                
+                for layer_obj in obj.getchildren():
+                    lobj_name = self.nstracker.local_name(layer_obj.tag)
+                    if lobj_name == "Namespace.ownedElement":
+                        self.uml_layer = layer_obj
+                        return
 
     def parse_uml(self):
         """Build a database based on UML description."""
@@ -54,13 +85,25 @@ class XmiParser(BaseParser):
                     # found database name
                     self.dbname = ret
         
+        # Check for database name
+        if self.dbname is None:
+            # no UML Class defined as Database, so lets set the default
+            # name to it.
+            self.dbname = DEFAULT_DB_NAME
+            
         self.adapt_data()
 
     def parse_table_from_uml_class(self, uml_obj):
         """Parse table attributes and operations."""
         
         def parse_operation(op_obj):
+            """Parse table operations."""
             op_name = op_obj.attrib['name']
+
+            if 'stereotype' not in op_obj.attrib:
+                # could raise error (bad operation)
+                return
+            
             op_stereotype = op_obj.attrib['stereotype'].lower()
             
             if op_obj.attrib['stereotype'] == 'index':
@@ -131,15 +174,15 @@ class XmiParser(BaseParser):
         self.tables[obj_id] = new_table
 
     def parse_table_association(self, uml_obj):
+        """Parse references between UML Classes."""
         ta_id = uml_obj.attrib['xmi.id']
         new_assoc = []
         for end in uml_obj.getchildren()[0].getchildren():
-            #new_assoc.append((end.attrib['xmi.id'], end.attrib['name'],
-            #                  end.attrib['type']))
             new_assoc.append((end.attrib['name'], end.attrib['type']))
         self.association[ta_id] = new_assoc
     
     def parse_table_inheritance(self, uml_obj):
+        """Parse generalizations between UML Classes."""
         self.generalization[uml_obj.attrib['xmi.id']] = (
             uml_obj.attrib['child'], uml_obj.attrib['parent'])
     
@@ -159,7 +202,9 @@ class XmiParser(BaseParser):
                 generalization = self.generalization[inherit]
                 if tid not in generalization:
                     # RunetimeError is not correct here, but I will leave it
-                    # for now (hopefully before a commit).
+                    # for now (hopefully before a commit). It seems
+                    # Umbrello doesn't even allow such bad creation for this
+                    # kind of object.
                     raise RuntimeError("Weird generalization error!")
                 
                 if generalization[0] == tid:
@@ -190,10 +235,19 @@ class XmiParser(BaseParser):
         for _, assoc in self.association.iteritems():
             tables, roles, conns = [], [], []
             for table_assoc in assoc:
+                if not table_assoc[0] or not table_assoc[1]:
+                    # could raise an error here (incomplete association)
+                    continue
+                
                 roles.append(table_assoc[0])
                 conns.append(table_assoc[1])
                 tables.append(self.tables[table_assoc[1]])
                 
+            if not tables or \
+               (not tables[0]['operations'] and not tables[1]['operations']):
+                # could raise an error here (invalid association)
+                continue
+            
             for oper in tables[0]['operations'][roles[0]]:
                 if oper[0] == 'fk':
                     tables[0]['references'][roles[0]] = (conns[1], roles[1])
